# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"Layers": "10_storage.ipynb",
         "raw_to_datastack": "10_storage.ipynb",
         "tree": "10_storage.ipynb",
         "underscorify": "10_storage.ipynb",
         "append": "10_storage.ipynb",
         "repack": "10_storage.ipynb",
         "max_and_sum_spectra": "10_storage.ipynb",
         "DataStack": "10_storage.ipynb",
         "DATASTACK_EXT": "10_storage.ipynb",
         "L": "10_storage.ipynb",
         "calibrate": "20_calibration.ipynb",
         "detector_angle": "20_calibration.ipynb",
         "find_instrument_peaks": "20_calibration.ipynb",
         "compton_shift": "20_calibration.ipynb",
         "RHODIUM_Ka": "20_calibration.ipynb",
         "IRON_Ka": "20_calibration.ipynb",
         "ImageRegistrationHelper": "30_spatial-verus-spectral.ipynb",
         "warp": "30_spatial-verus-spectral.ipynb",
         "xy_to_latlon": "30_spatial-verus-spectral.ipynb",
         "latlon_to_xy": "30_spatial-verus-spectral.ipynb",
         "HotmaxAtlas": "40_hotmax.ipynb",
         "compute_hotmax_spectra": "40_hotmax.ipynb",
         "compute_hotmax_noise": "40_hotmax.ipynb",
         "get_patterns": "50_peak-pattern-puzzle.ipynb",
         "colorize": "50_peak-pattern-puzzle.ipynb",
         "plot_ptrn": "50_peak-pattern-puzzle.ipynb",
         "plot_patterns": "50_peak-pattern-puzzle.ipynb",
         "plot_puzzle": "50_peak-pattern-puzzle.ipynb",
         "get_instrument_pattern": "50_peak-pattern-puzzle.ipynb",
         "add_hotlines": "50_peak-pattern-puzzle.ipynb",
         "all_elements": "50_peak-pattern-puzzle.ipynb",
         "eoi": "50_peak-pattern-puzzle.ipynb",
         "get_continuum": "60_peakmaps.ipynb",
         "get_gaussians": "60_peakmaps.ipynb",
         "fit_spectrum": "60_peakmaps.ipynb",
         "get_spectral_slices": "60_peakmaps.ipynb",
         "compute_nmf_peakmaps": "60_peakmaps.ipynb",
         "multi_plot": "60_peakmaps.ipynb",
         "compute_nmf_element_maps": "60_peakmaps.ipynb",
         "gaussian_convolve": "90_xphysics.ipynb",
         "ElementLines": "90_xphysics.ipynb",
         "get_element_spectrum": "90_xphysics.ipynb",
         "get_element_spectra": "90_xphysics.ipynb",
         "UploadDir": "91_interactive-plotting.ipynb",
         "make_filenames": "91_interactive-plotting.ipynb",
         "make_gridbox_widget": "91_interactive-plotting.ipynb"}

modules = ["storage.py",
           "calibration.py",
           "warp.py",
           "hotmax.py",
           "peakpuzzle.py",
           "peakmaps.py",
           "xphysics.py",
           "interactive_visualization.py"]

doc_url = "https://fligt.github.io/maxrf4u/"

git_url = "https://github.com/fligt/maxrf4u/tree/main/"

def custom_doc_links(name): return None
