# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/maxrf4u',
                'doc_host': 'https://fligt.github.io',
                'git_url': 'https://github.com/fligt/maxrf4u',
                'lib_path': 'maxrf4u'},
  'syms': { 'maxrf4u.absorption': {},
            'maxrf4u.calibration': { 'maxrf4u.calibration.calibrate': ('calibration.html#calibrate', 'maxrf4u/calibration.py'),
                                     'maxrf4u.calibration.compton_shift': ('calibration.html#compton_shift', 'maxrf4u/calibration.py'),
                                     'maxrf4u.calibration.detector_angle': ('calibration.html#detector_angle', 'maxrf4u/calibration.py'),
                                     'maxrf4u.calibration.find_instrument_peaks': ( 'calibration.html#find_instrument_peaks',
                                                                                    'maxrf4u/calibration.py')},
            'maxrf4u.fingerprints': {},
            'maxrf4u.hotmax': { 'maxrf4u.hotmax.HotmaxAtlas': ('hotmax.html#hotmaxatlas', 'maxrf4u/hotmax.py'),
                                'maxrf4u.hotmax.HotmaxAtlas.__init__': ('hotmax.html#hotmaxatlas.__init__', 'maxrf4u/hotmax.py'),
                                'maxrf4u.hotmax.HotmaxAtlas.plot_spectra': ('hotmax.html#hotmaxatlas.plot_spectra', 'maxrf4u/hotmax.py'),
                                'maxrf4u.hotmax.HotmaxAtlas.plot_spectrum': ('hotmax.html#hotmaxatlas.plot_spectrum', 'maxrf4u/hotmax.py'),
                                'maxrf4u.hotmax._baseline': ('hotmax.html#_baseline', 'maxrf4u/hotmax.py'),
                                'maxrf4u.hotmax._group_superimposed': ('hotmax.html#_group_superimposed', 'maxrf4u/hotmax.py'),
                                'maxrf4u.hotmax._noiseline': ('hotmax.html#_noiseline', 'maxrf4u/hotmax.py'),
                                'maxrf4u.hotmax._untilt': ('hotmax.html#_untilt', 'maxrf4u/hotmax.py'),
                                'maxrf4u.hotmax.compute_hotmax_noise': ('hotmax.html#compute_hotmax_noise', 'maxrf4u/hotmax.py'),
                                'maxrf4u.hotmax.compute_hotmax_spectra': ('hotmax.html#compute_hotmax_spectra', 'maxrf4u/hotmax.py'),
                                'maxrf4u.hotmax.compute_subpeaks': ('hotmax.html#compute_subpeaks', 'maxrf4u/hotmax.py')},
            'maxrf4u.interactive_visualization': { 'maxrf4u.interactive_visualization.UploadDir': ( 'interactive-plotting.html#uploaddir',
                                                                                                    'maxrf4u/interactive_visualization.py'),
                                                   'maxrf4u.interactive_visualization.UploadDir.__init__': ( 'interactive-plotting.html#uploaddir.__init__',
                                                                                                             'maxrf4u/interactive_visualization.py'),
                                                   'maxrf4u.interactive_visualization.UploadDir.export_interactive_html': ( 'interactive-plotting.html#uploaddir.export_interactive_html',
                                                                                                                            'maxrf4u/interactive_visualization.py'),
                                                   'maxrf4u.interactive_visualization.UploadDir.imsave': ( 'interactive-plotting.html#uploaddir.imsave',
                                                                                                           'maxrf4u/interactive_visualization.py'),
                                                   'maxrf4u.interactive_visualization.UploadDir.listdir': ( 'interactive-plotting.html#uploaddir.listdir',
                                                                                                            'maxrf4u/interactive_visualization.py'),
                                                   'maxrf4u.interactive_visualization.UploadDir.listurls': ( 'interactive-plotting.html#uploaddir.listurls',
                                                                                                             'maxrf4u/interactive_visualization.py'),
                                                   'maxrf4u.interactive_visualization.make_filenames': ( 'interactive-plotting.html#make_filenames',
                                                                                                         'maxrf4u/interactive_visualization.py'),
                                                   'maxrf4u.interactive_visualization.make_gridbox_widget': ( 'interactive-plotting.html#make_gridbox_widget',
                                                                                                              'maxrf4u/interactive_visualization.py')},
            'maxrf4u.iplotting': {},
            'maxrf4u.peak_pattern_puzzle_solver': {},
            'maxrf4u.peakmaps': { 'maxrf4u.peakmaps._add_hotlines_ticklabels': ( 'peakmaps.html#_add_hotlines_ticklabels',
                                                                                 'maxrf4u/peakmaps.py'),
                                  'maxrf4u.peakmaps._fit_gaussian': ('peakmaps.html#_fit_gaussian', 'maxrf4u/peakmaps.py'),
                                  'maxrf4u.peakmaps._gaussian': ('peakmaps.html#_gaussian', 'maxrf4u/peakmaps.py'),
                                  'maxrf4u.peakmaps._get_spectral_slices': ('peakmaps.html#_get_spectral_slices', 'maxrf4u/peakmaps.py'),
                                  'maxrf4u.peakmaps.compute_nmf_element_maps': ( 'peakmaps.html#compute_nmf_element_maps',
                                                                                 'maxrf4u/peakmaps.py'),
                                  'maxrf4u.peakmaps.compute_nmf_peakmaps': ('peakmaps.html#compute_nmf_peakmaps', 'maxrf4u/peakmaps.py'),
                                  'maxrf4u.peakmaps.fit_spectrum_peaks': ('peakmaps.html#fit_spectrum_peaks', 'maxrf4u/peakmaps.py'),
                                  'maxrf4u.peakmaps.get_continuum': ('peakmaps.html#get_continuum', 'maxrf4u/peakmaps.py'),
                                  'maxrf4u.peakmaps.get_gaussians': ('peakmaps.html#get_gaussians', 'maxrf4u/peakmaps.py'),
                                  'maxrf4u.peakmaps.multi_plot': ('peakmaps.html#multi_plot', 'maxrf4u/peakmaps.py')},
            'maxrf4u.peakpuzzle': { 'maxrf4u.peakpuzzle.Peak_Pattern_Atlas': ( 'peak-pattern-puzzle.html#peak_pattern_atlas',
                                                                               'maxrf4u/peakpuzzle.py'),
                                    'maxrf4u.peakpuzzle.Peak_Pattern_Atlas.__init__': ( 'peak-pattern-puzzle.html#peak_pattern_atlas.__init__',
                                                                                        'maxrf4u/peakpuzzle.py'),
                                    'maxrf4u.peakpuzzle.Peak_Pattern_Atlas.plot_patterns': ( 'peak-pattern-puzzle.html#peak_pattern_atlas.plot_patterns',
                                                                                             'maxrf4u/peakpuzzle.py'),
                                    'maxrf4u.peakpuzzle.Peak_Pattern_Atlas.plot_puzzle': ( 'peak-pattern-puzzle.html#peak_pattern_atlas.plot_puzzle',
                                                                                           'maxrf4u/peakpuzzle.py'),
                                    'maxrf4u.peakpuzzle.Peak_Pattern_Atlas.update_eoi': ( 'peak-pattern-puzzle.html#peak_pattern_atlas.update_eoi',
                                                                                          'maxrf4u/peakpuzzle.py'),
                                    'maxrf4u.peakpuzzle.colorize': ('peak-pattern-puzzle.html#colorize', 'maxrf4u/peakpuzzle.py'),
                                    'maxrf4u.peakpuzzle.get_eoi_patterns': ( 'peak-pattern-puzzle.html#get_eoi_patterns',
                                                                             'maxrf4u/peakpuzzle.py'),
                                    'maxrf4u.peakpuzzle.get_instrument_pattern': ( 'peak-pattern-puzzle.html#get_instrument_pattern',
                                                                                   'maxrf4u/peakpuzzle.py'),
                                    'maxrf4u.peakpuzzle.plot_patterns': ('peak-pattern-puzzle.html#plot_patterns', 'maxrf4u/peakpuzzle.py'),
                                    'maxrf4u.peakpuzzle.update_eoi': ('peak-pattern-puzzle.html#update_eoi', 'maxrf4u/peakpuzzle.py')},
            'maxrf4u.speckles': {},
            'maxrf4u.storage': { 'maxrf4u.storage.DataStack': ('storage.html#datastack', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.DataStack.__init__': ('storage.html#datastack.__init__', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.DataStack.close': ('storage.html#datastack.close', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.DataStack.latest': ('storage.html#datastack.latest', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.DataStack.read': ('storage.html#datastack.read', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.DataStack.read_list': ('storage.html#datastack.read_list', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.DataStack.tree': ('storage.html#datastack.tree', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.DataStack.update_attrs': ('storage.html#datastack.update_attrs', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.Layers': ('storage.html#layers', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.Layers.__init__': ('storage.html#layers.__init__', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage._straighten': ('storage.html#_straighten', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage._unstraighten': ('storage.html#_unstraighten', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.append': ('storage.html#append', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.append_list': ('storage.html#append_list', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.make_raw_preview': ('storage.html#make_raw_preview', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.max_and_sum_spectra': ('storage.html#max_and_sum_spectra', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.raw_to_datastack': ('storage.html#raw_to_datastack', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.repack': ('storage.html#repack', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.tree': ('storage.html#tree', 'maxrf4u/storage.py'),
                                 'maxrf4u.storage.underscorify': ('storage.html#underscorify', 'maxrf4u/storage.py')},
            'maxrf4u.warp': { 'maxrf4u.warp.ImageRegistrationHelper': ( 'spatial-verus-spectral.html#imageregistrationhelper',
                                                                        'maxrf4u/warp.py'),
                              'maxrf4u.warp.ImageRegistrationHelper.__init__': ( 'spatial-verus-spectral.html#imageregistrationhelper.__init__',
                                                                                 'maxrf4u/warp.py'),
                              'maxrf4u.warp.ImageRegistrationHelper.get_marker_coordinates': ( 'spatial-verus-spectral.html#imageregistrationhelper.get_marker_coordinates',
                                                                                               'maxrf4u/warp.py'),
                              'maxrf4u.warp.ImageRegistrationHelper.show': ( 'spatial-verus-spectral.html#imageregistrationhelper.show',
                                                                             'maxrf4u/warp.py'),
                              'maxrf4u.warp._img_to_base64_url': ('spatial-verus-spectral.html#_img_to_base64_url', 'maxrf4u/warp.py'),
                              'maxrf4u.warp.latlon_to_xy': ('spatial-verus-spectral.html#latlon_to_xy', 'maxrf4u/warp.py'),
                              'maxrf4u.warp.warp': ('spatial-verus-spectral.html#warp', 'maxrf4u/warp.py'),
                              'maxrf4u.warp.xy_to_latlon': ('spatial-verus-spectral.html#xy_to_latlon', 'maxrf4u/warp.py')},
            'maxrf4u.xphysics': { 'maxrf4u.xphysics.ElementLines': ('xphysics.html#elementlines', 'maxrf4u/xphysics.py'),
                                  'maxrf4u.xphysics.ElementLines.__init__': ('xphysics.html#elementlines.__init__', 'maxrf4u/xphysics.py'),
                                  'maxrf4u.xphysics.gaussian_convolve': ('xphysics.html#gaussian_convolve', 'maxrf4u/xphysics.py'),
                                  'maxrf4u.xphysics.get_element_spectra': ('xphysics.html#get_element_spectra', 'maxrf4u/xphysics.py'),
                                  'maxrf4u.xphysics.get_element_spectrum': ('xphysics.html#get_element_spectrum', 'maxrf4u/xphysics.py')}}}
